package cn.wustlinghang.wusthelper.internal.graduate.exception;

import cn.wustlinghang.wusthelper.rpc.exception.RpcException;

import java.util.StringJoiner;

public class GraduateRpcException extends RpcException {
    public static final int UNDERGRAD_MODULE = 1;

    public GraduateRpcException(TypeCode typeCode,
                                SubModuleCode subModuleCode,
                                ErrorCode errorCode) {
        super(UNDERGRAD_MODULE,
                typeCode.ordinal(),
                subModuleCode.ordinal() * 100 + errorCode.ordinal(),
                new StringJoiner("/")
                        .add("GRADUATE")
                        .add(typeCode.name())
                        .add(subModuleCode.name())
                        .add(errorCode.name())
                        .toString()
        );
    }

    /**
     * 异常类型编码
     */
    public enum TypeCode {
        // 网络异常类型
        NETWORK_EXCEPTION,
        // 参数异常类型
        PARAM_EXCEPTION,
        // 权限认证异常类型
        AUTH_EXCEPTION,
        // 网页解析异常类型
        PARSE_EXCEPTION,
        // 其他的异常类型
        OTHER_EXCEPTION
    }

    /**
     * 子模块编码
     */
    public enum SubModuleCode {
        COMMON, AUTH, COURSE_TABLE, SCORE, STUDENT_INFO, TRAINING_PLAN
    }

    /**
     * 具体错误编码
     */
    public enum ErrorCode {
        REQUEST_INVALID,
        PARAM_INVALID,
        COOKIE_INVALID,
        NETWORK_ERROR,
        PARSE_ERROR,

        // 需要评教
        NEED_EVALUATE,

        AUTH_PASSWORD_WRONG,
        AUTH_CAPTCHA_WRONG,
        AUTH_UNKNOWN_ERROR
    }
}
