package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.parser.graduate.GraduateCourseTableParser;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateCourseTableApiService;
import cn.wustlinghang.mywust.data.global.Course;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.graduate.exception.GraduateRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class CourseTableService extends BaseService {

    private final RequestClientOption requestClientOption;

    private final GraduateCourseTableApiService courseTableApiService;
    private final GraduateCourseTableParser courseTableParser;

    public CourseTableService(RequestClientOption requestClientOption,
                              GraduateCourseTableApiService courseTableApiService,
                              GraduateCourseTableParser courseTableParser) {
        this.requestClientOption = requestClientOption;

        this.courseTableApiService = courseTableApiService;
        this.courseTableParser = courseTableParser;
    }

    public String getCourseTable(String cookie) throws RpcException {
        try {
            return courseTableApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.COURSE_TABLE);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.COURSE_TABLE,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public List<Course> parseCourseTable(String data) throws GraduateRpcException {
        try {
            return courseTableParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.COURSE_TABLE,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
