package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.parser.undergraduate.UndergradCourseTableParser;
import cn.wustlinghang.mywust.core.request.service.undergraduate.UndergradCourseTableApiService;
import cn.wustlinghang.mywust.data.global.Course;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.wusthelper.internal.undergrad.exception.UndergradRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class CourseTableService extends BaseService {
    private final UndergradCourseTableParser courseTableParser;
    private final UndergradCourseTableApiService courseTableApiService;

    public CourseTableService(UndergradCourseTableParser courseTableParser,
                              UndergradCourseTableApiService courseTableApiService) {
        this.courseTableParser = courseTableParser;
        this.courseTableApiService = courseTableApiService;
    }

    public String getCourseTable(String cookie, String term) throws RpcException {
        try {
            return courseTableApiService.getPage(term, cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, UndergradRpcException.SubModuleCode.COURSE_TABLE);
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.COURSE_TABLE,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public List<Course> parseCourseTable(String data) throws UndergradRpcException {
        try {
            if (data.contains("评教")) {
                throw new UndergradRpcException(
                        UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                        UndergradRpcException.SubModuleCode.COURSE_TABLE,
                        UndergradRpcException.ErrorCode.NEED_EVALUATE
                );
            }

            return courseTableParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.COURSE_TABLE,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
